using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace Jiraiya
{
    public class Station: MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "Jiraiya";
        public string pluginName = "Station";
        public string pluginVersion = "1";
        public string pluginDate = "09/08" + "/2021";
        public string pluginDescription = @"Script to control assets in the Scarestation VaR";
        #endregion

        #region Vars    



//        public GameObject player;
        public GameObject Ks_Station;
	public Atom player;
	public JSONStorable playerCylinder;
	public Transform navigationRig;
	public Vector3 oldPos;
	public Vector3 newPos;
	public Vector3 difPos;


	protected JSONStorableFloat Door006Pos;
	protected JSONStorableFloat Door007Pos;
	protected JSONStorableFloat Door019Pos;
	protected JSONStorableFloat Door020Pos;

	private string objectNameSearchString = "scifilevel";
	public Transform ObjectRoot;

	public JSONStorableBool _SystemsOnCheckbox;
	public JSONStorableBool _playerCollisionCheckbox;
	public JSONStorableBool playerCollision;

	public JSONStorable screenStorable;
	public JSONStorableBool screenOn;

	public Atom Door006Collision;

	public JSONStorable Door006CollisionStorable;
	public JSONStorableBool Door006CollisionOn;
	public JSONStorable Door019CollisionStorable;
	public JSONStorableBool Door019CollisionOn;


        public Transform Door019;
	public bool Door019Open=true;
	public JSONStorableBool _Door019checkbox;
	public Vector3 Door019OpenPosition;
        public Transform Door020;
	public bool Door020Open=true;
	public Vector3 Door020OpenPosition;

        public Transform Door006;
	public bool Door006Open=true;
	public JSONStorableBool _Door006checkbox;
	public Vector3 Door006OpenPosition;
        public Transform Door007;
	public bool Door007Open=true;
	public Vector3 Door007OpenPosition;

	protected float timer = 0f;
	protected float targetValue;
	protected float currentValue;

        private bool editorInitialized = false;
        private int initLoops = 0;
        private int maxInitLoops = 3600; //3600 for one minute

        private static Atom currentAtom;
        private static string currentAtomName = "None";
        private static string tempAtomName = "None";

        #endregion

	private JSONStorableBool CreateBoolCheckbox(string name, bool defaultValue)
	        {
	        JSONStorableBool storable = new JSONStorableBool(name, defaultValue);
	        storable.storeType = JSONStorableParam.StoreType.Full;
	        RegisterBool(storable);
	        CreateToggle(storable);
	        return storable;
	        }

        public override void Init()
        {
            try
            {
		if (containingAtom.type != "CustomUnityAsset")
	                {
	                SuperController.LogError("Station - This script is ONLY for the scifimap.assetbundle CUA ( script disabled).");
  	                return;
  		        }

		_SystemsOnCheckbox = CreateBoolCheckbox("Systems On", false);
		_Door019checkbox = CreateBoolCheckbox("Open Door 019", false);
		_Door006checkbox = CreateBoolCheckbox("Open Door 006", false);
		_playerCollisionCheckbox = CreateBoolCheckbox("Player collision", false);


//		playerCylinder = GetAtomById("ISCylinder")?.GetStorableById("position");
		screenStorable = GetAtomById("InvisibleLight#3")?.GetStorableByID("Light");
		screenOn = screenStorable.GetBoolJSONParam("on");


//		player=GetAtomById("[CameraRig]");
		navigationRig = SuperController.singleton.navigationRig;

                JSONStorableString helpText = new JSONStorableString("Help",
                    "<color=#000><size=35><b>Help</b></size></color>\n\n" +
                    "<color=#333>" +
                    "This script controls changeable aspects of the SciFi spacestation map.\n\n" +
                    "It is only intended for use with the scifimap.assetbundle in Scarestation VaR.\n\n" +
                    "</color>" + navigationRig.position);
               UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 500.0f;

//		navigationRig.position=transform.position + new Vector3(-12.2f,0.1f,64.2f);
//		navigationRig.position=new Vector3(-12.2f,0.1f,64.2f);
		oldPos=navigationRig.position;

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }


/*	foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
		{
		Atom atom = SuperController.singleton.GetAtomByUid (atomUID);

//		if (atom.type == "InvisibleLight" && atomUID=="[CameraRig]")
		if (atomUID=="[CameraRig]")
			{	// InvisibleLight#3 AtomControl on

			SuperController.LogError("Station - Atom found "+atomUID);
//			SuperController.LogError("FreeControllerV3[0] "+atom.freeControllers[0].name);
//			foreach (Rigidbody rb in atom.rigidbodies) if (rb.name.Contains("control")) SuperController.LogError("Rigidbody "+rb.name);
//			foreach (FreeControllerV3 rb in atom.freeControllers) SuperController.LogError("FreeControllerV3 "+rb.name);

//			LightOn = atom.GetStorableByID("on");
//			SuperController.LogError(atom.GetStorableByID("on").name);			

			}
		}
*/



        }



        // Update is called with each rendered frame by Unity
        void Update()
        {
	if( initLoops < maxInitLoops && editorInitialized == false )
		{
		initLoops++;
		InitEditor();
		}
	if( initLoops == maxInitLoops )
		{
		SuperController.LogError("Station : This CUA could not be initialized. Incompatible object.");
		initLoops++; // To avoid spamming the log with the error
		}


//	playerCylinder.transform=navigationRig.position;
	if (_playerCollisionCheckbox.val==false)
		{
		oldPos=newPos;
		newPos=navigationRig.position;
		}
	else
		{
		difPos=newPos-oldPos;
		difPos=difPos*10;
		navigationRig.position=navigationRig.position-difPos;
		_playerCollisionCheckbox.val=false;
		}

	if (UnityEngine.Random.Range(0, 8000)<200) screenOn.val=false; else screenOn.val=true;

	currentValue+=Time.deltaTime;
	if (currentValue >= targetValue)
		{
		targetValue = UnityEngine.Random.Range(0, 4);
		currentValue=0;

//		screenOn.val=!screenOn.val;
		}

	if (_Door019checkbox.val!=Door019Open&&_SystemsOnCheckbox.val) Door019Toggle();
	if (_Door006checkbox.val!=Door006Open&&_SystemsOnCheckbox.val) Door006Toggle();
        }






        private bool InitEditor()
	    {
            if( editorInitialized == true ) return true;
        
            ObjectRoot = getObjectRoot( containingAtom.reParentObject );
            if( ObjectRoot == null ) return false;
        
            Door019 = ObjectRoot.Find("Level 1/Box019");
            Door020 = ObjectRoot.Find("Level 1/Box020");
            Door006 = ObjectRoot.Find("Level 1/Box006");
            Door007 = ObjectRoot.Find("Level 1/Box007");
                
            if( Door019 && Door020)
		{
//		Get starting positions for doors because they are open
		Door019OpenPosition=Door019.transform.position;
		Door020OpenPosition=Door020.transform.position;
		Door006OpenPosition=Door006.transform.position;
		Door007OpenPosition=Door007.transform.position;
//		Shut the doors because that's how we want them for the game
		Door006Toggle();
		Door019Toggle();

//		Door006CollisionStorable=GetAtomById("InvisibleLight#3")?.GetStorableByID("Light");
//		Door006Collision=GetAtomById("CollisionTrigger");
//		Door006Collision.on=true;

		editorInitialized = true;                           
		return true;
		}
	    else
		{
                SuperController.LogError("Station : your CUA is missing mandatory components ( CUA Transforms ).");
                initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
                return false;
		}
            return false;
            }

     private Transform getObjectRoot( Transform parent )
	{
	foreach( Transform child in parent )
		{
                if( child.name.Contains(objectNameSearchString) ) return child;
                else
			{
			Transform childSearch = getObjectRoot(child);
			if (childSearch != null) return childSearch;
			}
		}
	return null;
        }

	void Door019Toggle()
	{
	Door019Open=_Door019checkbox.val;
	if (Door019Open)
		{
		Door019.transform.position=Door019OpenPosition;
		Door020.transform.position=Door020OpenPosition;
		}
	else
		{
		Door019.transform.position= Door019OpenPosition+new Vector3(1.1f,0,0);
		Door020.transform.position= Door020OpenPosition-new Vector3(1.25f,0,0);
		}
	}

	void Door006Toggle()
	{
	Door006Open=_Door006checkbox.val;
	if (Door006Open)
		{
		Door006.transform.position=Door006OpenPosition;
		Door007.transform.position=Door007OpenPosition;
		}
	else
		{
		Door006.transform.position= Door006OpenPosition+new Vector3(1.1f,0,0);
		Door007.transform.position= Door007OpenPosition-new Vector3(1.25f,0,0);
		}
	}
    }
}